@@ -13,7 +13,7 @@ void alac_free(alac_file *alac);
 
 struct alac_file {
   unsigned char *input_buffer;
-  int input_buffer_bitaccumulator; /* used so we can do arbitary
+  int input_buffer_bitaccumulator; /* used so we can do arbitrary
                                       bit reads */
 
   int samplesize;
