@@ -82,56 +82,90 @@ void mpris_metadata_watcher(struct metadata_bundle *argc, __attribute__((unused)
     debug(1, "This should never happen.");
   }
 
-  GVariantBuilder *dict_builder, *aa;
-
-  /* Build the metadata array */
-  // debug(1,"Build metadata");
-  dict_builder = g_variant_builder_new(G_VARIANT_TYPE("a{sv}"));
-
-  // Make up the artwork URI if we have one
-  if (argc->cover_art_pathname) {
-    char artURIstring[1024];
-    snprintf(artURIstring, sizeof(artURIstring), "file://%s", argc->cover_art_pathname);
-    // debug(1,"artURI String: \"%s\".",artURIstring);
-    GVariant *artUrl = g_variant_new("s", artURIstring);
-    g_variant_builder_add(dict_builder, "{sv}", "mpris:artUrl", artUrl);
-  }
-
-  // Add the TrackID if we have one
-  // Build the Track ID from the 16-byte item_composite_id in hex prefixed by
-  // /org/gnome/ShairportSync
-  char st[33];
-  char *pt = st;
-  int it;
-  int non_zero = 0;
-  if (argc->track_metadata) {
+  /*
+    // Add the TrackID if we have one
+    // Build the Track ID from the 16-byte item_composite_id in hex prefixed by
+    // /org/gnome/ShairportSync
+    char st[33];
+    char *pt = st;
+    int it;
+    int non_zero = 0;
     for (it = 0; it < 16; it++) {
       if (argc->track_metadata->item_composite_id[it])
         non_zero = 1;
       snprintf(pt, 3, "%02X", argc->track_metadata->item_composite_id[it]);
       pt += 2;
     }
-  }
-  *pt = 0;
+    *pt = 0;
 
-  if (non_zero) {
-    // debug(1, "Set ID using composite ID: \"0x%s\".", st);
-    char trackidstring[1024];
-    snprintf(trackidstring, sizeof(trackidstring), "/org/gnome/ShairportSync/%s", st);
-    GVariant *trackid = g_variant_new("o", trackidstring);
-    g_variant_builder_add(dict_builder, "{sv}", "mpris:trackid", trackid);
-  } else if ((argc->track_metadata) && (argc->track_metadata->item_id)) {
+    if (non_zero) {
+      // debug(1, "Set ID using composite ID: \"0x%s\".", st);
+      char trackidstring[1024];
+      snprintf(trackidstring, sizeof(trackidstring), "/org/gnome/ShairportSync/%s", st);
+      GVariant *trackid = g_variant_new("o", trackidstring);
+      g_variant_builder_add(dict_builder, "{sv}", "mpris:trackid", trackid);
+    } else if ((argc->track_metadata) && (argc->track_metadata->item_id)) {
+      char trackidstring[128];
+      // debug(1, "Set ID using mper ID: \"%u\".",argc->item_id);
+      snprintf(trackidstring, sizeof(trackidstring), "/org/gnome/ShairportSync/mper_%u",
+               argc->track_metadata->item_id);
+      GVariant *trackid = g_variant_new("o", trackidstring);
+      g_variant_builder_add(dict_builder, "{sv}", "mpris:trackid", trackid);
+    }
+
+  */
+
+  // Build the metadata array
+  debug(2, "Build metadata");
+  GVariantBuilder *dict_builder = g_variant_builder_new(G_VARIANT_TYPE("a{sv}"));
+
+  // Add in the artwork URI if it exists.
+  if (argc->cover_art_pathname) {
+    GVariant *artUrl = g_variant_new("s", argc->cover_art_pathname);
+    g_variant_builder_add(dict_builder, "{sv}", "mpris:artUrl", artUrl);
+  }
+
+  // Add in the Track ID based on the 'mper' metadata if it is non-zero
+  if (argc->item_id != 0) {
     char trackidstring[128];
-    // debug(1, "Set ID using mper ID: \"%u\".",argc->item_id);
     snprintf(trackidstring, sizeof(trackidstring), "/org/gnome/ShairportSync/mper_%u",
-             argc->track_metadata->item_id);
+             argc->item_id);
     GVariant *trackid = g_variant_new("o", trackidstring);
     g_variant_builder_add(dict_builder, "{sv}", "mpris:trackid", trackid);
   }
 
-  // Add the track length if it's non-zero
-  if ((argc->track_metadata) && (argc->track_metadata->songtime_in_milliseconds)) {
-    uint64_t track_length_in_microseconds = argc->track_metadata->songtime_in_milliseconds;
+  // Add the track name if it exists
+  if (argc->track_name) {
+    GVariant *track_name = g_variant_new("s", argc->track_name);
+    g_variant_builder_add(dict_builder, "{sv}", "xesam:title", track_name);
+  }
+
+  // Add the album name if it exists
+  if (argc->album_name) {
+    GVariant *album_name = g_variant_new("s", argc->album_name);
+    g_variant_builder_add(dict_builder, "{sv}", "xesam:album", album_name);
+  }
+
+  // Add the artist name if it exists
+  if (argc->artist_name) {
+    GVariantBuilder *artist_as = g_variant_builder_new(G_VARIANT_TYPE("as"));
+    g_variant_builder_add(artist_as, "s", argc->artist_name);
+    GVariant *artists = g_variant_builder_end(artist_as);
+    g_variant_builder_unref(artist_as);
+    g_variant_builder_add(dict_builder, "{sv}", "xesam:artist", artists);
+  }
+
+  // Add the genre if it exists
+  if (argc->genre) {
+    GVariantBuilder *genre_as = g_variant_builder_new(G_VARIANT_TYPE("as"));
+    g_variant_builder_add(genre_as, "s", argc->genre);
+    GVariant *genre = g_variant_builder_end(genre_as);
+    g_variant_builder_unref(genre_as);
+    g_variant_builder_add(dict_builder, "{sv}", "xesam:genre", genre);
+  }
+
+  if (argc->songtime_in_milliseconds) {
+    uint64_t track_length_in_microseconds = argc->songtime_in_milliseconds;
     track_length_in_microseconds *= 1000; // to microseconds in 64-bit precision
                                           // Make up the track name and album name
     // debug(1, "Set tracklength to %lu.", track_length_in_microseconds);
@@ -139,48 +173,9 @@ void mpris_metadata_watcher(struct metadata_bundle *argc, __attribute__((unused)
     g_variant_builder_add(dict_builder, "{sv}", "mpris:length", tracklength);
   }
 
-  // Add the track name if there is one
-  if ((argc->track_metadata) && (argc->track_metadata->track_name)) {
-    // debug(1, "Track name set to \"%s\".", argc->track_name);
-    GVariant *trackname = g_variant_new("s", argc->track_metadata->track_name);
-    g_variant_builder_add(dict_builder, "{sv}", "xesam:title", trackname);
-  }
-
-  // Add the album name if there is one
-  if ((argc->track_metadata) && (argc->track_metadata->album_name)) {
-    // debug(1, "Album name set to \"%s\".", argc->album_name);
-    GVariant *albumname = g_variant_new("s", argc->track_metadata->album_name);
-    g_variant_builder_add(dict_builder, "{sv}", "xesam:album", albumname);
-  }
-
-  // Add the artists if there are any (actually there will be at most one, but put it in an array)
-  if ((argc->track_metadata) && (argc->track_metadata->artist_name)) {
-    /* Build the artists array */
-    // debug(1,"Build artist array");
-    aa = g_variant_builder_new(G_VARIANT_TYPE("as"));
-    g_variant_builder_add(aa, "s", argc->track_metadata->artist_name);
-    GVariant *artists = g_variant_builder_end(aa);
-    g_variant_builder_unref(aa);
-    g_variant_builder_add(dict_builder, "{sv}", "xesam:artist", artists);
-  }
-
-  // Add the genres if there are any (actually there will be at most one, but put it in an array)
-  if ((argc->track_metadata) && (argc->track_metadata->genre)) {
-    // debug(1,"Build genre");
-    aa = g_variant_builder_new(G_VARIANT_TYPE("as"));
-    g_variant_builder_add(aa, "s", argc->track_metadata->genre);
-    GVariant *genres = g_variant_builder_end(aa);
-    g_variant_builder_unref(aa);
-    g_variant_builder_add(dict_builder, "{sv}", "xesam:genre", genres);
-  }
-
   GVariant *dict = g_variant_builder_end(dict_builder);
   g_variant_builder_unref(dict_builder);
-
-  // debug(1,"Set metadata");
   media_player2_player_set_metadata(mprisPlayerPlayerSkeleton, dict);
-
-  // media_player2_player_set_volume(mprisPlayerPlayerSkeleton, metadata_store.speaker_volume);
 }
 
 static gboolean on_handle_quit(MediaPlayer2 *skeleton, GDBusMethodInvocation *invocation,
