@@ -40,11 +40,11 @@ void on_properties_changed(__attribute__((unused)) GDBusProxy *proxy, GVariant *
   }
 }
 
-void notify_loudness_filter_active_callback(ShairportSync *proxy,
+void notify_loudness_callback(ShairportSync *proxy,
                                             __attribute__((unused)) gpointer user_data) {
-  //  printf("\"notify_loudness_filter_active_callback\" called with a gpointer of
+  //  printf("\"notify_loudness_callback\" called with a gpointer of
   //  %lx.\n",(int64_t)user_data);
-  gboolean ebl = shairport_sync_get_loudness_filter_active(proxy);
+  gboolean ebl = shairport_sync_get_loudness(proxy);
   if (ebl == TRUE)
     printf("Client reports loudness is enabled.\n");
   else
@@ -175,6 +175,15 @@ int main(int argc, char *argv[]) {
   sleep(5);
   shairport_sync_advanced_remote_control_call_set_volume(
       SHAIRPORT_SYNC_ADVANCED_REMOTE_CONTROL(proxy4), 60, NULL, NULL, 0);
+        
+  sleep(5);
+  g_print("Volume up for five seconds...\n");
+  shairport_sync_remote_control_call_volume_up(SHAIRPORT_SYNC_REMOTE_CONTROL(proxy3), NULL, NULL, NULL);
+
+  sleep(5);
+  g_print("Volume down\n");
+  shairport_sync_remote_control_call_volume_down(SHAIRPORT_SYNC_REMOTE_CONTROL(proxy3), NULL, NULL, NULL);
+  
   /*
   // sleep(1);
     shairport_sync_set_loudness_filter_active(SHAIRPORT_SYNC(proxy), TRUE);
